# ifndef __PSO__H
# define __PSO__H
# include <problem.h>

/*	Available PSO types
 *	=============================================================
 *	1. LDWPSO	The original PSO method.
 *	2. CENTERPSO	The proposed PSO variant  in Neurocomputing.
 *	3. GPSO		The proposed PSO variant by me.
 *	4. PPSO		The Parsopoulos proposed scheme for PSO.
	5. DPSO		The dynamic     pso scheme.
 *	=============================================================
 * */
# define	LDWPSO		1
# define	CENTERPSO	2
# define	GPSO		3
# define	PPSO		4
# define	DPSO		5

class Pso
{
	private:
		Problem *problem;
		int		pso_type;
        int     generations;
		Data		xbest;
		Data		lmargin;
		Data		rmargin;
		Data		center;
		double		ybest;
		int 		generation;
        vector<Data>	particle;
        vector<Data>	bestparticle;
		vector<Data>	vel;
		int		bestIndex;
		Data		fitness;
		Data		bestFitness;
		Data		oldBestFitness;
		void		calcFitnessArray();
		double		getInertia(int index);
		void		boundVelocity(double &v,int index);
		void		updateBest();
		void		updateCenter();
	public:
		Pso(Problem *p,int count,int type);
        void    setGenerations(int g);
        int     getGenerations() const;
        void	Solve(int iprint);
		void	nextGeneration();
		int	getGeneration();
        int	getparticleCount();
		int	getDimension();
		void	getBest(Data &x,double &y);
		~Pso();
};
# endif
